package facturation.logique;

import java.sql.SQLException;
import java.util.List;

import facturation.dao.ArticleDAO;
import facturation.dao.ClientDAO;
import facturation.dao.FactureDAO;
import facturation.modele.Article;
import facturation.modele.Client;
import facturation.modele.Facture;

/**
 * Classe gérant la logique générale de l'application.
 * On pourrait scinder cette classe en plusieurs parties (en gros, la création de nouveaux clients et la facturation sont à séparer).
 * 
 * <p> On rencontre ici, comme dans la plupart des applications, le problème suivant:
 * L'interface doit-elle connaître les objets du modèle? Par exemple, quand l'interface 
 * veut afficher les données du client, elle doit connaître son nom, son adresse, son id...
 * 
 * <p>Première solution: facturationLogique renvoie des objets "Client". L'interface utilisateur 
 * peut accéder à leurs champs pour les afficher. Le problème est qu'il est tentant pour le programmeur de 
 * modifier lesdits objets, et de s'attendre à voir ces modifications prises en compte. On perd alors la belle architecture 
 * en couche que nous avons proposé. On peut résoudre ce problème en documentant l'architecture en question, mais 
 * c'est une solution qui reste à la merci des programmeurs. Il est aussi possible de renvoyer des objets non modifiables, 
 * mais cela implique a priori de dupliquer les classes du modèle... Certains programmeurs renvoient des copies des objets utilisés par FacturationLogique,
 * et non les objets eux-même. Bref, on dispose d'un certain nombre de solution, dont aucune n'est 
 * complètement satisfaisante.
 * 
 * <p> Deuxième solution: facturationLogique ne renvoie pas d'objet "client", mais donne accès à leurs champs. C'est une solution qui allourdit considérablement
 * la couche applicative (mais le code peut être généré automatiquement par eclipse). Elle est assez complexe à utiliser quand le résultat est composite (quand c'est, par exemple, 
 * une liste).
 * 
 * <p> Le langage C++ propose une solution intéressante, en ayant la notion d'objets "contants", i.e. non modifiables.
 * 
 * <p> Nous choisissons dans cette application de renvoyer directement les objets du modèle. ON IMPOSE AU PROGRAMMEUR DE NE PAS MODIFIER LESDITS OBJECTS DEPUIS L'INTERFACE.
 * TOUTE MODIFICATION DOIT PASSER PAR L'APPEL DE MÉTHODES SUR FACTURATIONLOGIQUE.
 *  
 * @author rosmord
 *
 */
public class FacturationLogique {
	private ClientDAO clientDAO;
	private FactureDAO factureDAO;
	private ArticleDAO articleDAO;
	
	private Client clientCourant = new Client();
	private Facture factureCourante;

	public FacturationLogique(ClientDAO clientDAO, ArticleDAO articleDAO, FactureDAO factureDAO) {
		this.clientDAO = clientDAO;
		this.articleDAO= articleDAO;
		this.factureDAO= factureDAO;
	}

	/**
	 * Prépare le travail sur un nouveau client.
	 */
	public void reinitialiserClient() {
		clientCourant= new Client();
	}
	
	public void fixerNomClient(String nom) {
		clientCourant.setNom(nom);
	}

	public void fixerAdresseClient(String adresse) {
		clientCourant.setAdresse(adresse);
	}

	public void creerClient() {
		try {
			clientCourant.setIdClient(clientDAO.getNextId());
			clientDAO.create(clientCourant);
		} catch (SQLException e) {
			throw new ErreurDeSerialisationException(e);
		}
		
	}
	
	public List<Client> chercherClientParNom(String nom) {
		try {
			return clientDAO.findClientByNom(nom);
		} catch (SQLException e) {
			throw new ErreurDeSerialisationException(e);
		}
	}


	public Client getClientCourant() {
		return clientCourant;
	}
	
	public List<Article> getCatalogue() {
		try {
			return articleDAO.findAllArticles();
		} catch (SQLException e) {
			throw new ErreurDeSerialisationException(e);
		}
	}

	public void selectionnerClient(int id) {
		try {
			clientCourant= clientDAO.findClientById(id);
		} catch (SQLException e) {
			throw new ErreurDeSerialisationException(e);
		}
	}

	/**
	 * Ajoute un produit à la facture en cours.
	 * @param id
	 */
	public void ajouterProduit(int id) {
		try {
			Article article= articleDAO.findArticlebyId(id);
			factureCourante.addArticle(article);
		} catch (SQLException e) {
			throw new ErreurDeSerialisationException(e);
		}
		
	}

	/**
	 * Valide et sauvegarde la facture courante.
	 * On applique ici les règles de gestion.
	 * À  chaque fois qu'un client donné réalise une commande, 
	 * il gagne 1 point de fidélité. 
	 * Si le client a 2 points de fidélité (la commande en cours non comprise), 
	 * le prix total de sa commande est réduit de 10%. 
	 * Son nombre de points de fidélités est alors remis à 0.
	 * [Remarque: en fait, il passe à 1 après la commande courante !]
	 * 
	 * Il serait intéressant de permettre un calcul de la réduction avant validation.
	 * Nous le laissons en exercice !
	 * 
	 * De même, gag: que se passe-t-il avec une facture vide ?
	 */
	public void validerFacture() {
		double total= 0;
		for (Article a: factureCourante.getArticles()) {
			total+= a.getPrixTTC();
		}
		if (clientCourant.getPointsDeFidelite() == 2) {
			clientCourant.setPointsDeFidelite(1); // Remise à 0 + ajout de la commande.
			total= total* 0.9;
		} else {
			clientCourant.setPointsDeFidelite(clientCourant.getPointsDeFidelite() + 1);
		}
		// Le client est mis à jour, le total est connu.
		// On termine de construire l'objet facture:
		factureCourante.setClient(getClientCourant());
		factureCourante.setPrixTotal(total);
		// On sauvegarde tout le monde.
		
		try {
			factureCourante.setIdFacture(factureDAO.getNextId());
			clientDAO.update(getClientCourant());
			factureDAO.save(factureCourante);
		} catch (SQLException e) {
			throw new ErreurDeSerialisationException(e);
		}
		
	}

	/**
	 * Retourne le prix total de la facture courante (après calcul !!)
	 * @return
	 */
	public double getTotalFacture() {
		return factureCourante.getPrixTotal();
	}

	/**
	 * 
	 */
	public void initialiserFacture() {
		factureCourante= new Facture();
	}
}
